@@ -584,6 +584,8 @@ tls13_ClientSetupESNI(sslSocket *ss)
     const sslNamedGroupDef *group = NULL;
     PRTime now = PR_Now() / PR_USEC_PER_SEC;
 
+    PORT_Assert(!ss->xtnData.esniPrivateKey);
+
     if (!ss->esniKeys) {
         return SECSuccess;
     }
@@ -721,8 +723,8 @@ tls13_ServerGetEsniAEAD(const sslSocket *ss, PRUint64 suite,
 }
 
 SECStatus
-tls13_ServerDecryptEsniXtn(const sslSocket *ss, PRUint8 *in, unsigned int inLen,
-                           PRUint8 *out, int *outLen, int maxLen)
+tls13_ServerDecryptEsniXtn(const sslSocket *ss, const PRUint8 *in, unsigned int inLen,
+                           PRUint8 *out, unsigned int *outLen, unsigned int maxLen)
 {
     sslReader rdr = SSL_READER(in, inLen);
     PRUint64 suite;
