@@ -50,8 +50,8 @@
 
 static PRBool pemInitialized = PR_FALSE;
 
-pemInternalObject **pem_objs;
-int pem_nobjs = 0;
+LIST_HEAD(pem_objs);
+long pem_nobjs = 0L;
 int token_needsLogin[NUM_SLOTS];
 NSSCKMDSlot *lastEventSlot;
 
@@ -182,12 +182,12 @@ GetCertFields(unsigned char *cert, int cert_length,
 }
 
 static CK_RV
-assignObjectID(pemInternalObject *o, int objid)
+assignObjectID(pemInternalObject *o, const long objid)
 {
-    char id[16];
+    char id[24];
     int len;
 
-    sprintf(id, "%d", objid);
+    sprintf(id, "%ld", objid);
     len = strlen(id) + 1;       /* zero terminate */
     o->id.size = len;
     o->id.data = NSS_ZAlloc(NULL, len);
@@ -195,6 +195,7 @@ assignObjectID(pemInternalObject *o, int objid)
         return CKR_HOST_MEMORY;
 
     memcpy(o->id.data, id, len);
+    o->objid = objid;
     return CKR_OK;
 }
 
@@ -202,7 +203,7 @@ static pemInternalObject *
 CreateObject(CK_OBJECT_CLASS objClass,
              pemObjectType type, SECItem * certDER,
              SECItem * keyDER, const char *filename,
-             int objid, CK_SLOT_ID slotID)
+             long objid, CK_SLOT_ID slotID)
 {
     pemInternalObject *o;
     SECItem subject;
@@ -226,17 +227,17 @@ CreateObject(CK_OBJECT_CLASS objClass,
 
     switch (objClass) {
     case CKO_CERTIFICATE:
-        plog("Creating cert nick %s id %d in slot %ld\n", nickname, objid, slotID);
+        plog("Creating cert nick %s id %ld in slot %ld\n", nickname, objid, slotID);
         memset(&o->u.cert, 0, sizeof(o->u.cert));
         break;
     case CKO_PRIVATE_KEY:
-        plog("Creating key id %d in slot %ld\n", objid, slotID);
+        plog("Creating key id %ld in slot %ld\n", objid, slotID);
         memset(&o->u.key, 0, sizeof(o->u.key));
         /* more unique nicknames - https://bugzilla.redhat.com/689031#c66 */
         nickname = filename;
         break;
     case CKO_NETSCAPE_TRUST:
-        plog("Creating trust nick %s id %d in slot %ld\n", nickname, objid, slotID);
+        plog("Creating trust nick %s id %ld in slot %ld\n", nickname, objid, slotID);
         memset(&o->u.trust, 0, sizeof(o->u.trust));
         break;
     }
@@ -360,16 +361,12 @@ derEncodingsMatch(CK_OBJECT_CLASS objClass, pemInternalObject * obj,
 }
 
 static CK_RV
-LinkSharedKeyObject(int oldKeyIdx, int newKeyIdx)
+LinkSharedKeyObject(const long oldKeyIdx, const long newKeyIdx)
 {
-    int i;
-    for (i = 0; i < pem_nobjs; i++) {
+    pemInternalObject *obj;
+    list_for_each_entry(obj, &pem_objs, gl_list) {
         CK_RV rv;
-        pemInternalObject *obj = pem_objs[i];
-        if (NULL == obj)
-            continue;
-
-        if (atoi(obj->id.data) != oldKeyIdx)
+        if (obj->objid != oldKeyIdx)
             continue;
 
         NSS_ZFreeIf(obj->id.data);
@@ -381,13 +378,26 @@ LinkSharedKeyObject(int oldKeyIdx, int newKeyIdx)
     return CKR_OK;
 }
 
+/* return pointer to the internal object with given arrayIdx */
+static pemInternalObject *
+FindObjectByArrayIdx(const long arrayIdx)
+{
+    pemInternalObject *obj;
+
+    list_for_each_entry(obj, &pem_objs, gl_list)
+        if (arrayIdx == obj->arrayIdx)
+            return obj;
+
+    return NULL;
+}
+
 pemInternalObject *
 AddObjectIfNeeded(CK_OBJECT_CLASS objClass,
                   pemObjectType type, SECItem * certDER,
                   SECItem * keyDER, const char *filename,
-                  int objid, CK_SLOT_ID slotID, PRBool *pAdded)
+                  long objid, CK_SLOT_ID slotID, PRBool *pAdded)
 {
-    int i;
+    pemInternalObject *curObj;
 
     const char *nickname = strrchr(filename, '/');
     if (nickname
@@ -401,11 +411,7 @@ AddObjectIfNeeded(CK_OBJECT_CLASS objClass,
         *pAdded = PR_FALSE;
 
     /* first look for the object in pem_objs, it might be already there */
-    for (i = 0; i < pem_nobjs; i++) {
-        pemInternalObject *const curObj = pem_objs[i];
-        if (NULL == curObj)
-            continue;
-
+    list_for_each_entry(curObj, &pem_objs, gl_list) {
         /* Comparing DER encodings is dependable and frees the PEM module
          * from having to require clients to provide unique nicknames.
          */
@@ -419,11 +425,11 @@ AddObjectIfNeeded(CK_OBJECT_CLASS objClass,
              * the key object is shared by multiple client certificates, such
              * an assumption does not hold.  We have to update the references.
              */
-            LinkSharedKeyObject(pem_nobjs, i);
+            LinkSharedKeyObject(pem_nobjs, curObj->arrayIdx);
 
             if (CKO_CERTIFICATE == objClass) {
-                const int ref = atoi(curObj->id.data);
-                if (0 < ref && ref < pem_nobjs && !pem_objs[ref]) {
+                const long ref = curObj->objid;
+                if (0 < ref && ref < pem_nobjs && !FindObjectByArrayIdx(ref)) {
                     /* The certificate we are going to reuse refers to an
                      * object that has already been removed.  Make it refer
                      * to the object that will be added next (private key).
@@ -433,7 +439,8 @@ AddObjectIfNeeded(CK_OBJECT_CLASS objClass,
                 }
             }
 
-            plog("AddObjectIfNeeded: re-using internal object #%i\n", i);
+            plog("AddObjectIfNeeded: re-using internal object #%li\n",
+                 curObj->arrayIdx);
             curObj->refCount ++;
             return curObj;
         }
@@ -448,18 +455,9 @@ AddObjectIfNeeded(CK_OBJECT_CLASS objClass,
     /* initialize pointers to functions */
     pem_CreateMDObject(NULL, io, NULL);
 
-    io->gobjIndex = pem_nobjs;
-
-    /* add object to global array */
-    if (pem_objs)
-        pem_objs = NSS_ZRealloc(pem_objs,
-                (pem_nobjs + 1) * sizeof(pemInternalObject *));
-    else
-        pem_objs = NSS_ZNEWARRAY(NULL, pemInternalObject *, 1);
-    if (!pem_objs)
-        return NULL;
-    pem_objs[pem_nobjs] = io;
-    pem_nobjs++;
+    /* add object to global list */
+    io->arrayIdx = pem_nobjs++;
+    list_add_tail(&io->gl_list, &pem_objs);
 
     if (pAdded)
         *pAdded = PR_TRUE;
@@ -474,7 +472,8 @@ AddCertificate(char *certfile, char *keyfile, PRBool cacert,
 {
     pemInternalObject *o = NULL;
     CK_RV error = 0;
-    int objid, i = 0;
+    long objid;
+    int i = 0;
     SECItem **objs = NULL;
     char *ivstring = NULL;
     int cipher;
@@ -748,9 +747,8 @@ pem_Finalize
     if (!pemInitialized)
         return;
 
-    NSS_ZFreeIf(pem_objs);
-    pem_objs = NULL;
-    pem_nobjs = 0;
+    INIT_LIST_HEAD(&pem_objs);
+    pem_nobjs = 0L;
 
     PR_AtomicSet(&pemInitialized, PR_FALSE);
 }
